package cn.com.poc.agent_application.aggregate.impl;

import cn.com.poc.agent_application.aggregate.AgentApplicationInfoService;
import cn.com.poc.agent_application.constant.AgentApplicationConstants;
import cn.com.poc.agent_application.constant.AgentApplicationDialoguesRecordConstants;
import cn.com.poc.agent_application.constant.AgentApplicationGCConfigConstants;
import cn.com.poc.agent_application.entity.BizAgentApplicationGcConfigEntity;
import cn.com.poc.agent_application.entity.BizAgentApplicationInfoEntity;
import cn.com.poc.agent_application.entity.BizAgentApplicationPublishEntity;
import cn.com.poc.agent_application.entity.CreateAgentTitleAndDescEntity;
import cn.com.poc.agent_application.service.BizAgentApplicationGcConfigService;
import cn.com.poc.agent_application.service.BizAgentApplicationInfoService;
import cn.com.poc.agent_application.service.BizAgentApplicationPublishService;
import cn.com.poc.common.utils.BlContext;
import cn.com.poc.common.utils.JsonUtils;
import cn.com.poc.support.security.oauth.entity.UserBaseEntity;
import cn.com.poc.thirdparty.resource.demand.ai.aggregate.AICreateImageService;
import cn.com.poc.thirdparty.resource.demand.ai.aggregate.DemandKnowledgeService;
import cn.com.poc.thirdparty.resource.demand.ai.common.domain.*;
import cn.com.poc.thirdparty.resource.demand.ai.entity.generations.BaiduAISailsText2ImageRequest;
import cn.com.poc.thirdparty.resource.demand.ai.entity.generations.BaiduAISailsText2ImageResult;
import cn.com.poc.thirdparty.resource.demand.ai.entity.largemodel.LargeModelDemandResult;
import cn.com.poc.thirdparty.resource.demand.ai.entity.largemodel.LargeModelResponse;
import cn.com.poc.thirdparty.resource.demand.ai.function.*;
import cn.com.poc.thirdparty.service.LLMService;
import cn.com.yict.framemax.core.exception.BusinessException;
import cn.hutool.core.bean.BeanUtil;
import com.fasterxml.jackson.core.type.TypeReference;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang.ArrayUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.*;
import java.util.stream.Collectors;

@Component
public class AgentApplicationInfoServiceImpl implements AgentApplicationInfoService {

    final private String TEXT_EVENT_STREAM_CHARSET_UTF_8 = "text/event-stream;charset=UTF-8";

    final private String EVENT_STREAM_PREFIX = "data: ";

    final private Logger logger = LoggerFactory.getLogger(AgentApplicationInfoService.class);

    @Resource
    private BizAgentApplicationGcConfigService bizAgentApplicationGcConfigService;

    @Resource
    private BizAgentApplicationInfoService bizAgentApplicationInfoService;

    @Resource
    private BizAgentApplicationPublishService bizAgentApplicationPublishService;

    @Resource
    private DemandKnowledgeService demandKnowledgeService;

    @Resource
    private AICreateImageService aiCreateImageService;

    @Resource
    private LLMService llmService;


    @Override
    public boolean updateAndPublish(BizAgentApplicationInfoEntity bizAgentApplicationInfoEntity) throws Exception {
        UserBaseEntity userBaseEntity = BlContext.getCurrentUserNotException();
        Long userId = userBaseEntity.getUserId();
        bizAgentApplicationInfoEntity.setMemberId(userId.intValue());
        bizAgentApplicationInfoEntity.setAgentPublishStatus(AgentApplicationConstants.AGENT_PUBLISH_STATUS.PUBLISH);
        bizAgentApplicationInfoEntity.setPublishTime(new Date());
        if (null == bizAgentApplicationInfoService.update(bizAgentApplicationInfoEntity)) {
            logger.error("update and publish agent error , bizAgentApplicationInfoEntity:{}", bizAgentApplicationInfoEntity);
            throw new BusinessException("发布失败");
        }

        //发布应用
        bizAgentApplicationPublishService.deleteByAgentId(bizAgentApplicationInfoEntity.getAgentId());
        BizAgentApplicationPublishEntity publishEntity = new BizAgentApplicationPublishEntity();
        BeanUtil.copyProperties(bizAgentApplicationInfoEntity, publishEntity);
        bizAgentApplicationPublishService.save(publishEntity);
        return true;
    }

    @Override
    public String callAgentApplication(String largeModel, String[] unitIds, String agentSystem, String[] knowledgeIds, Integer communicationTurn, Float topP, List<Message> messages, Boolean isLongMemory, String[] variableStructure, String useStatus, HttpServletResponse httpServletResponse) throws Exception {
        logger.info("--------- Call Agent Application large model:{},unitIds:{},agentSystem:{},knowledgeIds:{}" + " communicationTurn:{},topP:{},messages:{}--------------", largeModel, unitIds, agentSystem, knowledgeIds, communicationTurn, topP, messages);
        String promptTemplate = buildDialogsPrompt(messages, agentSystem, knowledgeIds);

        Message[] messageArray = buildMessages(messages, communicationTurn, promptTemplate);

        BufferedReader bufferedReader = this.invokeLLM(largeModel, messageArray, topP, toolsConfig(variableStructure, isLongMemory), useStatus, isLongMemory);

        textOutput(httpServletResponse, bufferedReader);

        return null;
    }

    private Tool[] toolsConfig(String[] variableStructure, boolean isLongMemory){
        // 配置tools
        Tool[] tools = new Tool[AgentApplicationConstants.TOOLS.FUNCTION_NUMBER];
        tools[0] = new Tool();
        tools[0].setType("function");
        // 根据变量名保存记忆方法
        AbstractLargeModelFunction function = LargeModelFunctionEnum.valueOf("setValueMemory").getFunction();
        Map<String, Object> parameters = function.getParameters();
        Map<String, Object> contentName = new HashMap<>();
        contentName.put("type", "string");
        contentName.put("description", "内容名");
        contentName.put("enum", variableStructure); // 设置变量
        Map<String, Object> contentValue = new HashMap<>();
        contentValue.put("type", "string");
        contentValue.put("description", "内容值");
        Map<String, Object> properties = new HashMap<>();
        properties.put("contentName", contentName);
        properties.put("contentValue", contentValue);
        parameters.put("properties", properties);
        parameters.put("type","object");
        function.setParameters(parameters);
        tools[0].setFunction(function);

        // 根据变量名查询记忆方法
        AbstractLargeModelFunction searchFunction = LargeModelFunctionEnum.valueOf("search_memory_content_by_Enum").getFunction();
        Map<String, Object> searchParameters = function.getParameters();
        Map<String, Object> content = new HashMap<>();
        content.put("type", "string");
        content.put("description", "内容名");
        content.put("enum", variableStructure); // 设置变量
        Map<String, Object> searchProperties = new HashMap<>();
        searchProperties.put("content", content);
        searchParameters.put("properties", searchProperties);
        searchParameters.put("type","object");
        searchFunction.setParameters(searchParameters);
        tools[1] = new Tool();
        tools[1].setType("function");
        tools[1].setFunction(searchFunction);

        //todo 判断是否开启长期记忆的按钮，生成tools配置
        if (isLongMemory) {   //如果开启了
            tools[2] = new Tool();
            tools[2].setType("function");
            tools[2].setFunction(LargeModelFunctionEnum.valueOf("searchMemoryContent").getFunction());

            tools[3] = new Tool();
            tools[3].setType("function");
            tools[3].setFunction(LargeModelFunctionEnum.valueOf("setLongMemory").getFunction());
        }
        return tools;
    }

    @Override
    public void createAgentSystem(String input, HttpServletResponse httpServletResponse) throws Exception {
        BizAgentApplicationGcConfigEntity configEntity = bizAgentApplicationGcConfigService.getByConfigCode(AgentApplicationGCConfigConstants.AGENT_SYSTEM);
        if (null == configEntity || StringUtils.isBlank(configEntity.getConfigSystem())) {
            throw new BusinessException("创建[角色指令]配置不存在");
        }

        List<MultiContent> multiContents = new ArrayList<>();
        multiContents.add(new MultiContent() {{
            String configSystem = configEntity.getConfigSystem();
            configSystem = configSystem.replace("${input}", input);
            setText(configSystem);
            setType("text");
        }});
        Message message = new Message();
        message.setContent(multiContents);
        message.setRole(AgentApplicationDialoguesRecordConstants.ROLE.USER);
        List<Message> messages = new ArrayList<Message>() {{
            add(message);
        }};

        LargeModelResponse largeModelResponse = new LargeModelResponse();
        largeModelResponse.setModel(configEntity.getLargeModel());
        largeModelResponse.setMessages(messages.toArray(new Message[1]));
        largeModelResponse.setTopP(configEntity.getTopP());
        largeModelResponse.setUser("POC-CREATE-AGENT-SYSTEM");

        BufferedReader bufferedReader = llmService.chatChunk(largeModelResponse);
        textOutput(httpServletResponse, bufferedReader);
    }

    @Override
    public String createPreamble(String agentTitle, String agentDesc, String agentSystem) {
        BizAgentApplicationGcConfigEntity configEntity = bizAgentApplicationGcConfigService.getByConfigCode(AgentApplicationGCConfigConstants.AGENT_PREAMBLE);
        if (null == configEntity || StringUtils.isBlank(configEntity.getConfigSystem())) {
            throw new BusinessException("创建[开场白]配置不存在");
        }

        List<MultiContent> multiContents = new ArrayList<>();
        multiContents.add(new MultiContent() {{
            String configSystem = configEntity.getConfigSystem();
            configSystem = buildPreambleConfigSystem(configSystem, agentTitle, agentDesc, agentSystem);

            setText(configSystem);
            setType("text");
        }});
        Message message = new Message();
        message.setContent(multiContents);
        message.setRole(AgentApplicationDialoguesRecordConstants.ROLE.USER);
        List<Message> messages = new ArrayList<Message>() {{
            add(message);
        }};

        LargeModelResponse largeModelResponse = new LargeModelResponse();
        largeModelResponse.setModel(configEntity.getLargeModel());
        largeModelResponse.setMessages(messages.toArray(new Message[1]));
        largeModelResponse.setTopP(configEntity.getTopP());
        largeModelResponse.setUser("POC-CREATE-PREAMBLE");

        LargeModelDemandResult largeModelDemandResult = llmService.chat(largeModelResponse);
        if (largeModelDemandResult == null || !"0".equals(largeModelDemandResult.getCode())) {
            logger.error("create preamble error, largeModelResponse:{}", largeModelResponse);
            throw new BusinessException("创建[开场白]失败");
        }

        return largeModelDemandResult.getMessage();
    }

    private static String buildPreambleConfigSystem(String configSystem, String agentTitle, String agentDesc, String agentSystem) {
        //若 name  desc    prompt  都为空，传入默认参数  name  入参 智能问答机器人   desc  随时为你解答问题
        if (StringUtils.isBlank(agentSystem) && StringUtils.isBlank(agentTitle) && StringUtils.isBlank(agentDesc)) {
            return configSystem.replace("${agent_title}", "智能问答机器人").replace("${agent_desc}", " 随时为你解答问题");
        }
        //若 name  desc   其中一项不为空则入参  不传入prompt
        if (StringUtils.isNotBlank(agentTitle) || (StringUtils.isNotBlank(agentDesc))) {
            return configSystem.replace("${agent_title}", StringUtils.isNotBlank(agentTitle) ? agentTitle : StringUtils.EMPTY)
                    .replace("${agent_desc}", StringUtils.isNotBlank(agentDesc) ? agentDesc : StringUtils.EMPTY);
        }

        // 若 name  desc 都空   prompt 有内容，则把prompt 传入desc
        if (StringUtils.isNotBlank(agentSystem)) {
            return configSystem.replace("${agent_desc}", agentSystem).replace("${agent_title}", "智能问答机器人");
        }
        return configSystem;
    }

    @Override
    public List<String> createFeaturedQuestions(String agentTitle, String agentDesc) {
        BizAgentApplicationGcConfigEntity configEntity = bizAgentApplicationGcConfigService.getByConfigCode(AgentApplicationGCConfigConstants.AGENT_FEATURED_QUESTIONS);
        if (null == configEntity || StringUtils.isBlank(configEntity.getConfigSystem())) {
            throw new BusinessException("创建[开场白]配置不存在");
        }

        List<MultiContent> multiContents = new ArrayList<>();
        multiContents.add(new MultiContent() {{
            String configSystem = configEntity.getConfigSystem();
            configSystem = configSystem.replace("${agent_title}", StringUtils.isNotBlank(agentTitle) ? agentTitle : StringUtils.EMPTY);
            configSystem = configSystem.replace("${agent_desc}", StringUtils.isNotBlank(agentDesc) ? agentDesc : StringUtils.EMPTY);
            setText(configSystem);
            setType("text");
        }});
        Message message = new Message();
        message.setContent(multiContents);
        message.setRole(AgentApplicationDialoguesRecordConstants.ROLE.USER);
        List<Message> messages = new ArrayList<Message>() {{
            add(message);
        }};

        LargeModelResponse largeModelResponse = new LargeModelResponse();
        largeModelResponse.setModel(configEntity.getLargeModel());
        largeModelResponse.setMessages(messages.toArray(new Message[1]));
        largeModelResponse.setTopP(configEntity.getTopP());
        largeModelResponse.setUser("POC-FEATURED-QUESTIONS");

        LargeModelDemandResult largeModelDemandResult = llmService.chat(largeModelResponse);
        String res = largeModelDemandResult.getMessage();
        int start = res.lastIndexOf("[");
        int end = res.lastIndexOf("]");
        return JsonUtils.deSerialize(res.substring(start, end + 1), new TypeReference<List<String>>() {
        }.getType());
    }

    @Override
    public String createAgentIcon(String agentTitle, String agentDesc) throws Exception {

        BizAgentApplicationGcConfigEntity configEntity = bizAgentApplicationGcConfigService.getByConfigCode(AgentApplicationGCConfigConstants.AGENT_ICON);
        if (null == configEntity || StringUtils.isBlank(configEntity.getConfigSystem())) {
            throw new BusinessException("创建[头像提示语]配置不存在，请联系管理员");
        }

        List<MultiContent> multiContents = new ArrayList<>();
        multiContents.add(new MultiContent() {{
            String configSystem = configEntity.getConfigSystem();
            configSystem = configSystem.replace("${agent_title}", StringUtils.isNotBlank(agentTitle) ? agentTitle : StringUtils.EMPTY);
            configSystem = configSystem.replace("${agent_desc}", StringUtils.isNotBlank(agentDesc) ? agentDesc : StringUtils.EMPTY);
            setText(configSystem);
            setType("text");
        }});
        Message message = new Message();
        message.setContent(multiContents);
        message.setRole(AgentApplicationDialoguesRecordConstants.ROLE.USER);
        List<Message> messages = new ArrayList<Message>() {{
            add(message);
        }};

        LargeModelResponse largeModelResponse = new LargeModelResponse();
        largeModelResponse.setModel(configEntity.getLargeModel());
        largeModelResponse.setMessages(messages.toArray(new Message[1]));
        largeModelResponse.setTopP(configEntity.getTopP());
        largeModelResponse.setUser("POC-CREATE-AGENT-PROMPT");
        LargeModelDemandResult largeModelDemandResult = llmService.chat(largeModelResponse);

        if (largeModelDemandResult == null || !"0".equals(largeModelDemandResult.getCode()) || StringUtils.isEmpty(largeModelDemandResult.getMessage())) {
            logger.error("create agent icon prompt error, largeModelResponse:{}", largeModelResponse);
            throw new BusinessException("创建[头像]失败,请稍后再试");
        }

        logger.info("create agent application icon prompt, result:{}", largeModelDemandResult);

        String createIconPrompt = largeModelDemandResult.getMessage();
        BaiduAISailsText2ImageRequest baiduAISailsText2ImageRequest = new BaiduAISailsText2ImageRequest();
        baiduAISailsText2ImageRequest.setPrompt(createIconPrompt);
        baiduAISailsText2ImageRequest.setNegativePrompt(null);
        baiduAISailsText2ImageRequest.setSize("768x768");
        baiduAISailsText2ImageRequest.setN(1);
        baiduAISailsText2ImageRequest.setSteps(15);
        baiduAISailsText2ImageRequest.setSamplerIndex("Euler a");
        baiduAISailsText2ImageRequest.setSeed(null);
        baiduAISailsText2ImageRequest.setCfgCcale(null);
        baiduAISailsText2ImageRequest.setStyle("Base");
        baiduAISailsText2ImageRequest.setUserId("AGENT-APPLICATION-ICON-CREATE");
        baiduAISailsText2ImageRequest.setAcctId(null);

        BaiduAISailsText2ImageResult baiduAISailsText2ImageResult = aiCreateImageService.executeSailsText2Image(baiduAISailsText2ImageRequest);
        if (baiduAISailsText2ImageResult == null
                || CollectionUtils.isEmpty(baiduAISailsText2ImageResult.getData())
                || StringUtils.isEmpty(baiduAISailsText2ImageResult.getData().get(0).getUrl())) {
            logger.error("create agent icon error, baiduAISailsText2ImageRequest:{} ", baiduAISailsText2ImageRequest);
            throw new BusinessException("创建[头像]失败,请稍后再试");
        }
        return baiduAISailsText2ImageResult.getData().get(0).getUrl();
    }

    @Override
    public CreateAgentTitleAndDescEntity createAgentTitleAndDesc(String input) {
        BizAgentApplicationGcConfigEntity configEntity = bizAgentApplicationGcConfigService.getByConfigCode(AgentApplicationGCConfigConstants.AGENT_INFO);
        if (null == configEntity || StringUtils.isBlank(configEntity.getConfigSystem())) {
            throw new BusinessException("创建[应用信息]配置不存在，请联系管理员");
        }
        List<MultiContent> multiContents = new ArrayList<>();
        multiContents.add(new MultiContent() {{
            String configSystem = configEntity.getConfigSystem();
            configSystem = configSystem.replace("${input}", StringUtils.isNotBlank(input) ? input : StringUtils.EMPTY);
            setText(configSystem);
            setType("text");
        }});
        Message message = new Message();
        message.setContent(multiContents);
        message.setRole(AgentApplicationDialoguesRecordConstants.ROLE.USER);
        List<Message> messages = new ArrayList<Message>() {{
            add(message);
        }};

        LargeModelResponse largeModelResponse = new LargeModelResponse();
        largeModelResponse.setModel(configEntity.getLargeModel());
        largeModelResponse.setMessages(messages.toArray(new Message[1]));
        largeModelResponse.setTopP(configEntity.getTopP());
        largeModelResponse.setUser("POC-CREATE-AGENT-INFO");
        LargeModelDemandResult largeModelDemandResult = llmService.chat(largeModelResponse);

        if (largeModelDemandResult == null || !"0".equals(largeModelDemandResult.getCode()) || StringUtils.isEmpty(largeModelDemandResult.getMessage())) {
            logger.error("create agent icon title And desc error, largeModelResponse:{}", largeModelResponse);
            throw new BusinessException("创建[应用信息]失败,请稍后再试");
        }
        logger.info("create agent application icon title And desc, response:{},result:{}", largeModelResponse, largeModelDemandResult);
        String res = largeModelDemandResult.getMessage();
        int start = res.lastIndexOf("{");
        int end = res.lastIndexOf("}");

        return JsonUtils.deSerialize(res.substring(start, end + 1), CreateAgentTitleAndDescEntity.class);
    }

    private String buildDialogsPrompt(List<Message> messages, String agentSystem, String[] knowledgeIds) {
        String promptTemplate = bizAgentApplicationGcConfigService.getByConfigCode(AgentApplicationGCConfigConstants.AGENT_BASE_SYSTEM).getConfigSystem();
        promptTemplate = promptTemplate.replace("${agentSystem}", StringUtils.isNotBlank(agentSystem) ? agentSystem : StringUtils.EMPTY);
        // 调用知识库
        if (ArrayUtils.isNotEmpty(knowledgeIds)) {
            List<String> knowledgeResults = demandKnowledgeService.searchKnowledge(messages.get(messages.size() - 1).getContent().get(0).getText(), Arrays.stream(knowledgeIds).collect(Collectors.toList()), 10);
            promptTemplate = promptTemplate.replace("${knowledgeResults}", knowledgeResults.toString());
        }
        return promptTemplate;
    }

    /**
     * * 调用llm
     *
     * @param largeModel
     * @param messageArray
     * @param topP
     * @return
     * @throws Exception
     */
    private BufferedReader invokeLLM(String largeModel, Message[] messageArray, Float topP, Tool[] tools, String useStatus, Boolean isLongMemory) throws Exception {
        LargeModelResponse largeModelResponse = new LargeModelResponse();
        largeModelResponse.setModel(largeModel);
        largeModelResponse.setMessages(messageArray);
        largeModelResponse.setTopP(topP);
        largeModelResponse.setStream(true);
        largeModelResponse.setUser("POE");
        largeModelResponse.setTools(tools);
        largeModelResponse.setTool_choice("auto");
        BufferedReader bufferedReader = llmService.chatChunk(largeModelResponse);
        String res = "";
        bufferedReader.mark(200);
        LargeModelResponse modelResponse = new LargeModelResponse();
        modelResponse.setModel(largeModel);
        modelResponse.setTopP(topP);
        modelResponse.setStream(true);
        modelResponse.setUser("POE");
        modelResponse.setTools(tools);
        modelResponse.setTool_choice("auto");
        String functionResult = null;
        StringBuffer finishReason = new StringBuffer();
        StringBuffer functionName = new StringBuffer();
        StringBuffer functionArguments = new StringBuffer();

        boolean isFunctionCall = false;

        while ((res = bufferedReader.readLine()) != null) {
            if (StringUtils.isBlank(res)) {
                continue;
            }
            // 添加function 参数 LargeModelDemandResult 和中台接口一致
            LargeModelDemandResult result = JsonUtils.deSerialize(res.replaceFirst(EVENT_STREAM_PREFIX, StringUtils.EMPTY), LargeModelDemandResult.class);
            if (!"0".equals(result.getCode())) {
                logger.error("LLM Error,code:{}", result.getCode());
                BusinessException ex = new BusinessException("调用失败");
                throw ex;
            }

            if (!(result.getFunction() != null && (StringUtils.isNotBlank(result.getFunction().getName()) || StringUtils.isNotBlank(result.getFunction().getArguments()))) && !isFunctionCall) {
                bufferedReader.reset();
                break;
            } else {
                isFunctionCall = true;
                if (result.getFunction().getName() != null && !result.getFunction().getName().isEmpty()) {
                    functionName.append(result.getFunction().getName());
                }
                if (result.getFunction().getArguments() != null && !result.getFunction().getArguments().isEmpty()) {
                    functionArguments.append(result.getFunction().getArguments());
                }
            }

            if (result.getFinish_reason() != null) {
                finishReason.append(result.getFinish_reason());
            }
            // 走到了最后一个流，判断是否为function_call
            // 如果是function_call，则处理function
            if (finishReason.toString().equals("tool_calls")) {
                // 如果是function_call，则处理function
                if (!functionName.toString().isEmpty() && !functionArguments.toString().isEmpty()) {
                    // 执行函数返回结果
                    ExecuteLargeModelFunctionEnum functionEnum = ExecuteLargeModelFunctionEnum.valueOf(functionName.toString());
                    AbstractExecuteLargeModelFunction executeFunction = functionEnum.getExecuteFunction();
                    functionResult = executeFunction.doMemoryFunction(functionArguments.toString(), useStatus, isLongMemory);
                }
                if (functionResult != null) {
                    // 处理function - 1，获取function_call 2. 调用对应function 3. 返回function_call并且调用LLM 4.获取bufferedReader 返回
                    Message assistantMessage = new Message();
                    assistantMessage.setRole("assistant");
                    assistantMessage.setContent(null);
                    List<FunctionCall> functionCalls = new ArrayList<>();
                    FunctionCall functionCall = new FunctionCall();
                    functionCall.setName(functionName.toString());
                    functionCall.setArguments(functionArguments.toString());
                    functionCalls.add(functionCall);
                    assistantMessage.setFunction_call(functionCalls);

                    Message[] sendMessage = new Message[messageArray.length + 2];
                    for (int i = 0; i < messageArray.length; i++) {
                        sendMessage[i] = messageArray[i];
                    }
                    sendMessage[messageArray.length] = assistantMessage;

                    Message functionMessage = new Message();
                    functionMessage.setRole("function");
                    functionMessage.setName(functionName.toString());
                    List<MultiContent> content = new ArrayList<>();
                    MultiContent multiContent = new MultiContent();
                    multiContent.setText(functionResult);
                    multiContent.setType("text");
                    content.add(multiContent);
                    functionMessage.setContent(content);
                    sendMessage[messageArray.length + 1] = functionMessage;

                    modelResponse.setMessages(sendMessage);
                    return llmService.chatChunk(modelResponse);
                }
            }
        }
        // 若不为function_call，则直接返回bufferedReader
        return bufferedReader;

    }

    /**
     * 文本输出结果
     *
     * @param httpServletResponse
     * @param bufferedReader
     * @throws IOException
     */
    private String textOutput(HttpServletResponse httpServletResponse, BufferedReader bufferedReader) throws
            IOException {
        String res = "";
        httpServletResponse.setContentType(TEXT_EVENT_STREAM_CHARSET_UTF_8);
        PrintWriter writer = httpServletResponse.getWriter();
        StringBuilder output = new StringBuilder();
        while ((res = bufferedReader.readLine()) != null) {
            if (StringUtils.isBlank(res)) {
                continue;
            }
            LargeModelDemandResult result = JsonUtils.deSerialize(res.replaceFirst(EVENT_STREAM_PREFIX, StringUtils.EMPTY), LargeModelDemandResult.class);
            if (!"0".equals(result.getCode())) {
                logger.error("LLM Error,code:{}", result.getCode());
                BusinessException ex = new BusinessException("调用失败");
                writer.write(EVENT_STREAM_PREFIX + JsonUtils.serialize(ex) + "\n\n");
                writer.flush();
                throw ex;
            }
            output.append(result.getMessage());
            writer.write(res + "\n\n");
            writer.flush();
        }
        writer.write(EVENT_STREAM_PREFIX + "[DONE]\n\n");
        // 关闭资源
        bufferedReader.close();
        writer.close();
        return output.toString();
    }

    /**
     * 构建消息【大模型参数】
     *
     * @param messages
     * @param communicationTurn
     * @param promptTemplate
     * @return
     */
    private static Message[] buildMessages(List<Message> messages, Integer communicationTurn, String
            promptTemplate) {
        int messLength = messages.size() - 1;
        int skip = communicationTurn * 2;
        if (skip < messLength) {
            messages = messages.subList(messLength - skip, messages.size());
        }

        if (StringUtils.isNotBlank(promptTemplate)) {
            List<MultiContent> content = new ArrayList<>();
            MultiContent multiContent = new MultiContent();
            multiContent.setText(promptTemplate);
            multiContent.setType("text");
            content.add(multiContent);
            Message systemMessage = new Message();
            systemMessage.setRole(AgentApplicationDialoguesRecordConstants.ROLE.SYSTEM);
            systemMessage.setContent(content);
            messages.add(0, systemMessage);
        }
        return messages.toArray(new Message[0]);
    }
}
